# -*- coding: utf-8 -*-

import os
import sys
import unittest
import codecs
import io
import datetime
import json
from colbert.livre_journal import livre_journal_to_list
from decimal import Decimal

CURRENT_DIR = os.path.dirname(__file__)
VERSION_INFO = sys.version_info


class LivreJournalTestCase(unittest.TestCase):
    def test_check_livre_journal(self):
        from colbert.livre_journal import check_livre_journal
        livre_journal = codecs.open(os.path.join(CURRENT_DIR, "livre-journal.txt"),
                                    mode="r", encoding="utf-8")
        result = check_livre_journal(livre_journal)
        self.assertEqual(result, [
            ['18/03/2011 -  Frais de constitution de la société CFE Paris.',
             'OK : débit = crédit (90.45).'],
            ['18/03/2011 -  Frais de constitution de la société - Annonce légale.',
             'OK : débit = crédit (99.00).'],
            ['31/03/2011 -  Facture 2011-01 MyClient1',
             'OK : débit = crédit (5 980.00).'],
            ['01/04/2011 -  Résultat arrêté compte',
             'OK : débit = crédit (48.00).'],
            ['02/04/2011 -  Capital initial', 'OK : débit = crédit (1 500.00).'],
            ['04/04/2011 -  Facture 2011-02 MyClient2',
             'OK : débit = crédit (1 794.00).'],
            ['28/04/2011 -  Cotisation Option PRO  LCL',
             'OK : débit = crédit (15.00).'],
            ['02/05/2011 -  Abonnement LCL Access',
             'OK : débit = crédit (3.00).'],
            ['11/06/2011 -  BHV - Fournitures des bureau (Livres comptables).',
             'OK : débit = crédit (25.65).'],
            ['15/06/2011 -  Remise chèque XXXXXXX règlement facture 2011-02',
             'OK : débit = crédit (2 088.00).'],
            ['05/07/2011 -  Traitement mois de juin gérant.',
             'OK : débit = crédit (3 000.00).'],
            ['08/08/2011 -  Chèque XXXXXXY', 'OK : débit = crédit (393.00).'],
            ['02/09/2011 -  Virement MyClient1 ZZZZZZZZZZZ',
             'OK : débit = crédit (6 960.00).'],
            ['03/09/2011 -  Abonnement LCL Access',
             'OK : débit = crédit (3.00).'],
            ['28/09/2011 -  Facture 2011-04 MyClient1',
             'OK : débit = crédit (13 156.00).'],
            ['30/09/2011 -  Solde des comptes de TVA du 01/03/2011 au 30/09/2011',
             'OK : débit = crédit (1 274.00).'],
            ['06/10/2011 -  Chèque WWWWWWW',
             'OK : débit = crédit (1 240.00).'],
            ['01/11/2011 -  Facture 2011-05 MyClient1',
             'OK : débit = crédit (5 382.00).'],
            ['17/11/2011 -  Chèque ZZZZZZZ', 'OK : débit = crédit (402.00).'],
            ['01/12/2011 -  Abonnement LCL Access',
             'OK : débit = crédit (3.00).'],
            ['01/12/2011 -  Virement MyClient1 WWWWWWWWWW',
             'OK : débit = crédit (21 576.00).'],
            ['01/12/2011 -  Facture 2011-06 MyClient3',
             'OK : débit = crédit (8 372.00).'],
            ['31/12/2011 -  Solde des comptes de TVA du 01/10/2011 au 31/12/2011',
             'OK : débit = crédit (3 038.00).'],
            ['31/12/2011 -  Prestation MyClient1 décembre 2011',
             'OK : débit = crédit (13 156.00).'],
            ['01/01/2012 -  Prestation MyClient1 décembre 2011',
             'OK : débit = crédit (13 156.00).'],
            ['03/01/2012 -  Facture 2012-01 MyClient1',
             'OK : débit = crédit (13 156.00).'],
            ["01/02/2012 -  Restaurant La Tour d'argent",
             'OK : débit = crédit (49.80).']
        ])

    def test_check_ecriture_livre_journal(self):
        from colbert.livre_journal import check_ecriture_livre_journal
        ecriture = {
            'date': datetime.date(2011, 3, 18),
            'numero_ligne_debut': 13,
            'numero_ligne_fin': 17,
            'intitule': [' Frais de constitution de la société CFE Paris.'],
            'ecritures': [
                {
                    'credit': Decimal('0.00'),
                    'debit': Decimal('80.00'),
                    'nom_compte': "Achats - Frais d'actes et de contentieux",
                    'numero_compte_credit': '',
                    'numero_compte_debit': '6227'
                },
                {
                    'credit': Decimal('0.00'),
                    'debit': Decimal('10.45'),
                    'nom_compte': 'T.V.A. déductible sur autres biens et services',
                    'numero_compte_credit': '',
                    'numero_compte_debit': '44566'
                },
                {
                    'credit': Decimal('90.45'),
                    'debit': Decimal('0.00'),
                    'nom_compte': 'Associés - Comptes courants',
                    'numero_compte_credit': '455',
                    'numero_compte_debit': ''
                }
            ],
        }
        self.assertEqual(
            check_ecriture_livre_journal(ecriture),
            ['18/03/2011 -  Frais de constitution de la société CFE Paris.', 'OK : débit = crédit (90.45).']
        )

        # Erreur dans la colonne du compte.
        ecriture = {
            'date': datetime.date(2011, 3, 18),
            'numero_ligne_debut': 13,
            'numero_ligne_fin': 17,
            'intitule': [' Frais de constitution de la société CFE Paris.'],
            'ecritures': [
                {
                    'credit': Decimal('0.00'),
                    'debit': Decimal('80.00'),
                    'nom_compte': "Achats - Frais d'actes et de contentieux",
                    'numero_compte_credit': '',
                    'numero_compte_debit': '6227'
                },
                {
                    'credit': Decimal('0.00'),
                    'debit': Decimal('10.45'),
                    'nom_compte': 'T.V.A. déductible sur autres biens et services',
                    'numero_compte_credit': '',
                    'numero_compte_debit': '44566'
                },
                {
                    'credit': Decimal('90.45'),
                    'debit': Decimal('0.00'),
                    'nom_compte': 'Associés - Comptes courants',
                    'numero_compte_credit': '',
                    'numero_compte_debit': '455'
                }
            ],
        }
        self.assertEqual(
            check_ecriture_livre_journal(ecriture),
            ['18/03/2011 -  Frais de constitution de la société CFE Paris.',
             'ERREUR : incohérence entre les colonnes numéro de compte et montant']
        )

        ecriture = {
            'date': datetime.date(2011, 3, 18),
            'numero_ligne_debut': 13,
            'numero_ligne_fin': 17,
            'intitule': [' Frais de constitution de la société CFE Paris.'],
            'ecritures': [
                {
                    'credit': Decimal('0.00'),
                    'debit': Decimal('80.00'),
                    'nom_compte': "Achats - Frais d'actes et de contentieux",
                    'numero_compte_credit': '6227',
                    'numero_compte_debit': ''
                },
                {
                    'credit': Decimal('0.00'),
                    'debit': Decimal('10.45'),
                    'nom_compte': 'T.V.A. déductible sur autres biens et services',
                    'numero_compte_credit': '',
                    'numero_compte_debit': '44566'
                },
                {
                    'credit': Decimal('90.45'),
                    'debit': Decimal('0.00'),
                    'nom_compte': 'Associés - Comptes courants',
                    'numero_compte_credit': '455',
                    'numero_compte_debit': ''
                }
            ],
        }
        self.assertEqual(
            check_ecriture_livre_journal(ecriture),
            ['18/03/2011 -  Frais de constitution de la société CFE Paris.',
             'ERREUR : incohérence entre les colonnes numéro de compte et montant']
        )

    def test_ecritures_de_cloture(self):
        from colbert.livre_journal import ecritures_de_cloture
        balance_des_comptes = codecs.open(os.path.join(CURRENT_DIR, "balance_des_comptes-2011.json"),
                                          mode="r", encoding="utf-8")

        edc = ecritures_de_cloture(json.loads(balance_des_comptes.read()))

        self.maxDiff = None
        self.assertEqual(
            edc,
            [{'date': datetime.date(2011, 12, 31),
              'ecritures': [{'credit': Decimal('0.00'),
                             'debit': Decimal('40000.00'),
                             'nom_compte': 'Produits - prestations de services',
                             'numero_compte_credit': '',
                             'numero_compte_debit': '706'},
                            {'credit': Decimal('0.00'),
                             'debit': Decimal('0.34'),
                             'nom_compte': 'Produits divers de gestion courante',
                             'numero_compte_credit': '',
                             'numero_compte_debit': '758'},
                            {'credit': Decimal('40000.34'),
                             'debit': Decimal('0.0'),
                             'nom_compte': 'Regroupement des comptes de produits',
                             'numero_compte_credit': '127',
                             'numero_compte_debit': ''}],
              'intitule': ['Ecritures de clôture des comptes.']},
             {'date': datetime.date(2011, 12, 31),
              'ecritures': [{'credit': Decimal('0.0'),
                             'debit': Decimal('4048.44'),
                             'nom_compte': 'Regroupement des comptes de charges',
                             'numero_compte_credit': '',
                             'numero_compte_debit': '126'},
                            {'credit': Decimal('21.44'),
                             'debit': Decimal('0.00'),
                             'nom_compte': 'Achats - Fournitures de bureau',
                             'numero_compte_credit': '60225',
                             'numero_compte_debit': ''},
                            {'credit': Decimal('160.00'),
                             'debit': Decimal('0.00'),
                             'nom_compte': "Achats - Frais d'actes et de contentieux",
                             'numero_compte_credit': '6227',
                             'numero_compte_debit': ''},
                            {'credit': Decimal('72.00'),
                             'debit': Decimal('0.00'),
                             'nom_compte': 'Autres frais de commission sur prestations de services',
                             'numero_compte_credit': '6278-LCL',
                             'numero_compte_debit': ''},
                            {'credit': Decimal('3000.00'),
                             'debit': Decimal('0.00'),
                             'nom_compte': 'Charges - Salaires et appointements',
                             'numero_compte_credit': '6411',
                             'numero_compte_debit': ''},
                            {'credit': Decimal('393.00'),
                             'debit': Decimal('0.00'),
                             'nom_compte': 'Charges - cotisations RSI',
                             'numero_compte_credit': '6411-RSI',
                             'numero_compte_debit': ''},
                            {'credit': Decimal('161.80'),
                             'debit': Decimal('0.00'),
                             'nom_compte': 'Charges - cotisations URSSAF - Allocations familliales',
                             'numero_compte_credit': '6411-URSF1',
                             'numero_compte_debit': ''},
                            {'credit': Decimal('153.31'),
                             'debit': Decimal('0.00'),
                             'nom_compte': 'Charges - cotisations URSSAF - CSG/RDS déductible',
                             'numero_compte_credit': '6411-URSF2',
                             'numero_compte_debit': ''},
                            {'credit': Decimal('86.89'),
                             'debit': Decimal('0.00'),
                             'nom_compte': 'Charges - cotisations URSSAF - CSG/RDS non-déductible',
                             'numero_compte_credit': '6411-URSF3',
                             'numero_compte_debit': ''}],
              'intitule': ['Ecritures de clôture des comptes.']},
             {'date': datetime.date(2011, 12, 31),
              'ecritures': [{'credit': Decimal('0.0'),
                             'debit': Decimal('40000.34'),
                             'nom_compte': 'Regroupement des comptes de produits',
                             'numero_compte_credit': '',
                             'numero_compte_debit': '127'},
                            {'credit': Decimal('4048.44'),
                             'debit': Decimal('0.0'),
                             'nom_compte': 'Regroupement des comptes de charges',
                             'numero_compte_credit': '126',
                             'numero_compte_debit': ''},
                            {'credit': Decimal('35951.90'),
                             'debit': Decimal('0.0'),
                             'nom_compte': "résultat de l'exercice (bénéfice)",
                             'numero_compte_credit': '120',
                             'numero_compte_debit': ''}],
              'intitule': ["Enregistrement du résultat net de l'exercice"]}]
        )

    def test_ecritures_to_livre_journal(self):
        from colbert.livre_journal import ecritures_to_livre_journal
        ecritures = codecs.open(os.path.join(CURRENT_DIR, "ecritures_de_cloture-2011.json"),
                                mode="r", encoding="utf-8")
        output = io.StringIO()
        ecritures_to_livre_journal(json.loads(ecritures.read()), output)
        self.maxDiff = None
        self.assertEqual(output.getvalue(),
"""+---------------------------------------------------------------------------------------------------------------------------------------------------------+
| Ecritures pour le Livre-journal                                                                                                                         |
+=============+=================+=================+==============================================================+=================+=================+====+
|| 31/12/2011 ||                ||                || Ecritures de clôture des comptes.                           ||                ||                |    |
||            || 706            ||                || Produits - prestations de services                          || 40 000.00      ||                |    |
||            || 758            ||                || Produits divers de gestion courante                         || 0.34           ||                |    |
||            ||                || 127            ||     Regroupement des comptes de produits                    ||                || 40 000.34      |    |
+-------------+-----------------+-----------------+--------------------------------------------------------------+-----------------+-----------------+----+
|| 31/12/2011 ||                ||                || Ecritures de clôture des comptes.                           ||                ||                |    |
||            || 126            ||                || Regroupement des comptes de charges                         || 4 048.44       ||                |    |
||            ||                || 60225          ||     Achats - Fournitures de bureau                          ||                || 21.44          |    |
||            ||                || 6227           ||     Achats - Frais d'actes et de contentieux                ||                || 160.00         |    |
||            ||                || 6278-LCL       ||     Autres frais de commission sur prestations de services  ||                || 72.00          |    |
||            ||                || 6411           ||     Charges - Salaires et appointements                     ||                || 3 000.00       |    |
||            ||                || 6411-RSI       ||     Charges - cotisations RSI                               ||                || 393.00         |    |
||            ||                || 6411-URSF1     ||     Charges - cotisations URSSAF - Allocations familliales  ||                || 161.80         |    |
||            ||                || 6411-URSF2     ||     Charges - cotisations URSSAF - CSG/RDS déductible       ||                || 153.31         |    |
||            ||                || 6411-URSF3     ||     Charges - cotisations URSSAF - CSG/RDS non-déductible   ||                || 86.89          |    |
+-------------+-----------------+-----------------+--------------------------------------------------------------+-----------------+-----------------+----+
|| 31/12/2011 ||                ||                || Enregistrement du résultat net de l'exercice                ||                ||                |    |
||            || 127            ||                || Regroupement des comptes de produits                        || 40 000.34      ||                |    |
||            ||                || 126            ||     Regroupement des comptes de charges                     ||                || 4 048.44       |    |
||            ||                || 120            ||     résultat de l'exercice (bénéfice)                       ||                || 35 951.90      |    |
+-------------+-----------------+-----------------+--------------------------------------------------------------+-----------------+-----------------+----+

""")

    def test_get_solde_compte(self):
        from colbert.livre_journal import get_solde_compte

        livre_journal = codecs.open(os.path.join(CURRENT_DIR, "livre-journal.txt"),
                                    mode="r", encoding="utf-8")
        livre_journal_list = livre_journal_to_list(livre_journal)

        date_debut = datetime.date(2011, 1, 1)
        date_fin = datetime.date(2011, 12, 31)

        debit, credit = get_solde_compte(livre_journal_list, "512", date_debut, date_fin)
        # TODO verifier.
        self.assertEqual(debit, Decimal("22679.35"))
        self.assertEqual(credit, Decimal("0.00"))

    def test_livre_journal_to_list(self):
        from colbert.livre_journal import RX_DATE_INTITULE, RX_SUITE_INTITULE, RX_ECRITURE
        self.maxDiff = None

        # Première ligne d'écriture.
        s = "|| 31/03/2011 ||                ||                || Facture 2011-01 AdenClassifieds      ||          ||           |   | "
        if VERSION_INFO >= (2, 7):
            self.assertRegex(s, RX_DATE_INTITULE)
        m = RX_DATE_INTITULE.match(s)
        self.assertEqual(m.groupdict(), {'intitule': ' Facture 2011-01 AdenClassifieds      ',
                                         'credit': '           ',
                                         'debit': '          ',
                                         'date': '31/03/2011',
                                         'numero_compte_credit': '                ',
                                         'numero_compte_debit': '                ',
                                         'checked': '   '})

        # Ligne supplementaire d'intitulé.
        s = "||   ||       ||        ||    suite et fin de l'intitulé     ||     ||   |  |"
        if VERSION_INFO >= (2, 7):
            self.assertRegex(s, RX_SUITE_INTITULE)
        m = RX_SUITE_INTITULE.match(s)

        # Ecriture.
        s = "||   ||    4111-clie   ||        ||   Clients - ventes de biens ou prestations de services  ||    8 372   ||         |   |"
        if VERSION_INFO >= (2, 7):
            self.assertRegex(s, RX_ECRITURE)
        m = RX_ECRITURE.match(s)
        self.assertEqual(m.groupdict(), {'nom_compte': 'Clients - ventes de biens ou prestations de services  ',
                                         'credit': '         ',
                                         'debit': '    8 372   ',
                                         'date': '   ',
                                         'numero_compte_credit': '        ',
                                         'numero_compte_debit': '    4111-clie   ',
                                         'checked': '   '})

        # Conversion du livre journal.
        livre_journal = codecs.open(os.path.join(CURRENT_DIR, "livre-journal.txt"),
                                    mode="r", encoding="utf-8")
        livre_journal_list = livre_journal_to_list(livre_journal)
        self.assertEqual(livre_journal_list, [
            {'date': datetime.date(2011, 3, 18),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('80.00'),
                            'nom_compte': "Achats - Frais d'actes et de contentieux",
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6227'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('10.45'),
                            'nom_compte': 'T.V.A. déductible sur autres biens et services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44566'},
                           {'credit': Decimal('90.45'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Associés - Comptes courants',
                            'numero_compte_credit': '455',
                            'numero_compte_debit': ''}],
             'intitule': [' Frais de constitution de la société CFE Paris.'],
             'numero_ligne_debut': 13,
             'numero_ligne_fin': 17},
            {'date': datetime.date(2011, 3, 18),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('80.00'),
                            'nom_compte': "Achats - Frais d'actes et de contentieux MONTANT à vérifier",
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6227'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('19.00'),
                            'nom_compte': 'T.V.A. déductible sur autres biens et services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44566'},
                           {'credit': Decimal('99.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Associés - Comptes courants',
                            'numero_compte_credit': '455',
                            'numero_compte_debit': ''}],
             'intitule': [' Frais de constitution de la société - Annonce légale.'],
             'numero_ligne_debut': 18,
             'numero_ligne_fin': 22},
            {'date': datetime.date(2011, 3, 31),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('5980.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '4111-CL1'},
                           {'credit': Decimal('5000.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Produits - prestations de services',
                            'numero_compte_credit': '706',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('980.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '44587',
                            'numero_compte_debit': ''}],
             'intitule': [' Facture 2011-01 MyClient1',
                          '       Prestation MyClient1 mars 2011'],
             'numero_ligne_debut': 23,
             'numero_ligne_fin': 28},
            {'date': datetime.date(2011, 4, 1),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('48.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('48.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Résultat arrêté compte'],
             'numero_ligne_debut': 31,
             'numero_ligne_fin': 34},
            {'date': datetime.date(2011, 4, 2),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('1500.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '512'},
                           {'credit': Decimal('1500.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': "Capital et compte de l'exploitant",
                            'numero_compte_credit': '100',
                            'numero_compte_debit': ''}],
             'intitule': [' Capital initial',
                          '       Dépôt de 1500 € par Stanislas Guerra',
                          '       au LCL Ledru Rollin'],
             'numero_ligne_debut': 35,
             'numero_ligne_fin': 40},
            {'date': datetime.date(2011, 4, 4),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('1794.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '4111-CL2'},
                           {'credit': Decimal('1500.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Produits - prestations de services',
                            'numero_compte_credit': '706',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('294.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '44587',
                            'numero_compte_debit': ''}],
             'intitule': [' Facture 2011-02 MyClient2', '       Prestation MyClient2'],
             'numero_ligne_debut': 41,
             'numero_ligne_fin': 46},
            {'date': datetime.date(2011, 4, 28),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('15.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('15.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Cotisation Option PRO  LCL'],
             'numero_ligne_debut': 47,
             'numero_ligne_fin': 50},
            {'date': datetime.date(2011, 5, 2),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('3.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('3.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Abonnement LCL Access'],
             'numero_ligne_debut': 53,
             'numero_ligne_fin': 56},
            {'date': datetime.date(2011, 6, 11),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('21.44'),
                            'nom_compte': 'Achats - Fournitures de bureau',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '60225'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('4.21'),
                            'nom_compte': 'T.V.A. déductible sur autres biens et services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44566'},
                           {'credit': Decimal('25.65'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' BHV - Fournitures des bureau (Livres comptables).'],
             'numero_ligne_debut': 59,
             'numero_ligne_fin': 63},
            {'date': datetime.date(2011, 6, 15),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('1794.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '512'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('294.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44587'},
                           {'credit': Decimal('1794.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '4111-CL2',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('294.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'T.V.A. Collectée',
                            'numero_compte_credit': '44571',
                            'numero_compte_debit': ''}],
             'intitule': [' Remise chèque XXXXXXX règlement facture 2011-02'],
             'numero_ligne_debut': 64,
             'numero_ligne_fin': 69},
            {'date': datetime.date(2011, 7, 5),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('3000.00'),
                            'nom_compte': 'Charges - Salaires et appointements',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6411'},
                           {'credit': Decimal('3000.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Traitement mois de juin gérant.'],
             'numero_ligne_debut': 72,
             'numero_ligne_fin': 75},
            {'date': datetime.date(2011, 8, 8),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('393.00'),
                            'nom_compte': 'Charges - cotisations RSI',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6411-RSI'},
                           {'credit': Decimal('393.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Chèque XXXXXXY',
                          '     Cotisation trimestrielle RSI/Prévadiès.'],
             'numero_ligne_debut': 78,
             'numero_ligne_fin': 82},
            {'date': datetime.date(2011, 9, 2),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('5980.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '512'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('980.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44587'},
                           {'credit': Decimal('5980.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '4111-CL1',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('980.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'T.V.A. Collectée',
                            'numero_compte_credit': '44571',
                            'numero_compte_debit': ''}],
             'intitule': [' Virement MyClient1 ZZZZZZZZZZZ', '       Facture 2011-01'],
             'numero_ligne_debut': 85,
             'numero_ligne_fin': 91},
            {'date': datetime.date(2011, 9, 3),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('3.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('3.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Abonnement LCL Access'],
             'numero_ligne_debut': 92,
             'numero_ligne_fin': 95},
            {'date': datetime.date(2011, 9, 28),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('13156.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '4111-CL1'},
                           {'credit': Decimal('11000.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Produits - prestations de services',
                            'numero_compte_credit': '706',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('2156.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '44587',
                            'numero_compte_debit': ''}],
             'intitule': [' Facture 2011-04 MyClient1', '       Prestation aout 2011'],
             'numero_ligne_debut': 96,
             'numero_ligne_fin': 101},
            {'date': datetime.date(2011, 9, 30),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('1274.00'),
                            'nom_compte': 'TVA collecté',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44571'},
                           {'credit': Decimal('33.66'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'TVA déductible sur autres biens et services',
                            'numero_compte_credit': '44566',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('1240.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'TVA à décaisser',
                            'numero_compte_credit': '44551',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('0.34'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Produits divers de gestion courante',
                            'numero_compte_credit': '758',
                            'numero_compte_debit': ''}],
             'intitule': [' Solde des comptes de TVA du 01/03/2011 au 30/09/2011'],
             'numero_ligne_debut': 104,
             'numero_ligne_fin': 109},
            {'date': datetime.date(2011, 10, 6),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('1240.00'),
                            'nom_compte': 'TVA à décaisser',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44551'},
                           {'credit': Decimal('1240.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Chèque WWWWWWW',
                          '     Règlement de la TVA trimestrielle'],
             'numero_ligne_debut': 112,
             'numero_ligne_fin': 116},
            {'date': datetime.date(2011, 11, 1),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('5382.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '4111-CL1'},
                           {'credit': Decimal('4500.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Produits - prestations de services',
                            'numero_compte_credit': '706',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('882.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '44587',
                            'numero_compte_debit': ''}],
             'intitule': [' Facture 2011-05 MyClient1',
                          '       Prestation septembre 2011'],
             'numero_ligne_debut': 119,
             'numero_ligne_fin': 124},
            {'date': datetime.date(2011, 11, 17),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('161.80'),
                            'nom_compte': 'Charges - cotisations URSSAF - Allocations familliales',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6411-URSF1'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('153.31'),
                            'nom_compte': 'Charges - cotisations URSSAF - CSG/RDS déductible',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6411-URSF2'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('86.89'),
                            'nom_compte': 'Charges - cotisations URSSAF - CSG/RDS non-déductible',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6411-URSF3'},
                           {'credit': Decimal('402.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Chèque ZZZZZZZ',
                          '     Cotisation sociales Urssaf 4ème trimestre.'],
             'numero_ligne_debut': 125,
             'numero_ligne_fin': 131},
            {'date': datetime.date(2011, 12, 1),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('3.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('3.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Abonnement LCL Access'],
             'numero_ligne_debut': 134,
             'numero_ligne_fin': 137},
            {'date': datetime.date(2011, 12, 1),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('18538.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '512'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('2156.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44587'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('882.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44587'},
                           {'credit': Decimal('18538.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '4111-CL1',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('2156.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'T.V.A. Collectée',
                            'numero_compte_credit': '44571',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('882.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'T.V.A. Collectée',
                            'numero_compte_credit': '44571',
                            'numero_compte_debit': ''}],
             'intitule': [' Virement MyClient1 WWWWWWWWWW',
                          '       Facture 2011-04, 2011-05'],
             'numero_ligne_debut': 138,
             'numero_ligne_fin': 146},
            {'date': datetime.date(2011, 12, 1),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('8372.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '4111-CL3'},
                           {'credit': Decimal('7000.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Produits - prestations de services',
                            'numero_compte_credit': '706',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('1372.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '44587',
                            'numero_compte_debit': ''}],
             'intitule': [' Facture 2011-06 MyClient3'],
             'numero_ligne_debut': 147,
             'numero_ligne_fin': 151},
            {'date': datetime.date(2011, 12, 31),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('3038.00'),
                            'nom_compte': 'TVA collecté',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44571'},
                           {'credit': Decimal('3038.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'TVA à décaisser',
                            'numero_compte_credit': '44551',
                            'numero_compte_debit': ''}],
             'intitule': [' Solde des comptes de TVA du 01/10/2011 au 31/12/2011'],
             'numero_ligne_debut': 154,
             'numero_ligne_fin': 157},
            {'date': datetime.date(2011, 12, 31),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('13156.00'),
                            'nom_compte': 'Clients - Factures à établir',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '4181'},
                           {'credit': Decimal('11000.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Produits - prestations de services',
                            'numero_compte_credit': '706',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('2156.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '44587',
                            'numero_compte_debit': ''}],
             'intitule': [' Prestation MyClient1 décembre 2011'],
             'numero_ligne_debut': 160,
             'numero_ligne_fin': 164},
            {'date': datetime.date(2012, 1, 1),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('11000.00'),
                            'nom_compte': 'Produits - prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '706'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('2156.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44587'},
                           {'credit': Decimal('13156.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Clients - Factures à établir',
                            'numero_compte_credit': '4181',
                            'numero_compte_debit': ''}],
             'intitule': [' Prestation MyClient1 décembre 2011'],
             'numero_ligne_debut': 169,
             'numero_ligne_fin': 173},
            {'date': datetime.date(2012, 1, 3),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('13156.00'),
                            'nom_compte': 'Clients - ventes de biens ou prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '4111-CL1'},
                           {'credit': Decimal('11000.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Produits - prestations de services',
                            'numero_compte_credit': '706',
                            'numero_compte_debit': ''},
                           {'credit': Decimal('2156.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Taxes sur le CA sur factures à établir',
                            'numero_compte_credit': '44587',
                            'numero_compte_debit': ''}],
             'intitule': [' Facture 2012-01 MyClient1',
                          '       Prestation décembre 2011'],
             'numero_ligne_debut': 176,
             'numero_ligne_fin': 181},
            {'date': datetime.date(2012, 2, 1),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('46.02'),
                            'nom_compte': 'Charges - Réceptions',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6257'},
                           {'credit': Decimal('0.00'),
                            'debit': Decimal('3.78'),
                            'nom_compte': 'T.V.A. déductible sur autres biens et services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '44566'},
                           {'credit': Decimal('49.80'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [" Restaurant La Tour d'argent",
                          "       Déjeuner d'affaire avec Steve Jobs",
                          '       0.88€ TVA 19.6% ; 2.90€ TVA 7.0%'],
             'numero_ligne_debut': 184,
             'numero_ligne_fin': 190}
        ])

    def test_update_ecriture(self):
        from colbert.livre_journal import update_ecriture
        ecriture = {
            'date': "12/11/2014",
            'intitule': "Restaurant La Tour d'argent Déjeuner d'affaire avec Vladimir P.",
            'ecritures': [
                {'credit': '0.00',
                 'debit': '49.80',
                 'nom_compte': 'Charges - Réceptions',
                 'numero_compte_credit': '',
                 'numero_compte_debit': '6257'},
                {'credit': '49.80',
                 'debit': '0.00',
                 'nom_compte': 'Banques',
                 'numero_compte_credit': '512',
                 'numero_compte_debit': ''}
            ]
        }
        update_ecriture(ecriture, date="23/12/2014", montants=["33.40"])
        self.assertEqual(ecriture, {
            'date': "23/12/2014",
            'intitule': "Restaurant La Tour d'argent Déjeuner d'affaire avec Vladimir P.",
            'ecritures': [
                {'credit': '0.00',
                 'debit': '33.40',
                 'nom_compte': 'Charges - Réceptions',
                 'numero_compte_credit': '',
                 'numero_compte_debit': '6257'},
                {'credit': '33.40',
                 'debit': '0.00',
                 'nom_compte': 'Banques',
                 'numero_compte_credit': '512',
                 'numero_compte_debit': ''}
            ]
        })

        # Avec plusieurs montants.
        ecriture = {
            'date': "12/11/2014",
            'intitule': "Restaurant La Tour d'argent Déjeuner d'affaire avec Vladimir P.",
            'ecritures': [
                {'credit': '0.00',
                 'debit': '46.02',
                 'nom_compte': 'Charges - Réceptions',
                 'numero_compte_credit': '',
                 'numero_compte_debit': '6257'},
                {'credit': '0.00',
                 'debit': '3.78',
                 'nom_compte': 'T.V.A. déductible sur autres biens et services',
                 'numero_compte_credit': '',
                 'numero_compte_debit': '44566'},
                {'credit': '49.80',
                 'debit': '0.00',
                 'nom_compte': 'Banques',
                 'numero_compte_credit': '512',
                 'numero_compte_debit': ''}
            ]
        }
        update_ecriture(ecriture, date="23/12/2014", montants=["30.10", "3.30", "33.40"])
        self.assertEqual(ecriture, {
            'date': "23/12/2014",
            'intitule': "Restaurant La Tour d'argent Déjeuner d'affaire avec Vladimir P.",
            'ecritures': [
                {'credit': '0.00',
                 'debit': '30.10',
                 'nom_compte': 'Charges - Réceptions',
                 'numero_compte_credit': '',
                 'numero_compte_debit': '6257'},
                {'credit': '0.00',
                 'debit': '3.30',
                 'nom_compte': 'T.V.A. déductible sur autres biens et services',
                 'numero_compte_credit': '',
                 'numero_compte_debit': '44566'},
                {'credit': '33.40',
                 'debit': '0.00',
                 'nom_compte': 'Banques',
                 'numero_compte_credit': '512',
                 'numero_compte_debit': ''}
            ]
        })

    def test_rechercher_ecriture(self):
        from colbert.livre_journal import rechercher_ecriture
        livre_journal = codecs.open(os.path.join(CURRENT_DIR, "livre-journal.txt"),
                                    mode="r", encoding="utf-8")
        livre_journal_list = livre_journal_to_list(livre_journal)
        self.assertEqual(list(rechercher_ecriture("lcl", livre_journal_list)), [
            {'date': datetime.date(2011, 4, 2),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('1500.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '512'},
                           {'credit': Decimal('1500.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': "Capital et compte de l'exploitant",
                            'numero_compte_credit': '100',
                            'numero_compte_debit': ''}],
             'intitule': [' Capital initial',
                          '       Dépôt de 1500 € par Stanislas Guerra',
                          '       au LCL Ledru Rollin'],
             'numero_ligne_debut': 35,
             'numero_ligne_fin': 40},
            {'date': datetime.date(2011, 4, 28),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('15.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('15.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Cotisation Option PRO  LCL'],
             'numero_ligne_debut': 47,
             'numero_ligne_fin': 50},
            {'date': datetime.date(2011, 5, 2),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('3.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('3.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Abonnement LCL Access'],
             'numero_ligne_debut': 53,
             'numero_ligne_fin': 56},
            {'date': datetime.date(2011, 9, 3),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('3.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('3.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Abonnement LCL Access'],
             'numero_ligne_debut': 92,
             'numero_ligne_fin': 95},
            {'date': datetime.date(2011, 12, 1),
             'ecritures': [{'credit': Decimal('0.00'),
                            'debit': Decimal('3.00'),
                            'nom_compte': 'Autres frais de commission sur prestations de services',
                            'numero_compte_credit': '',
                            'numero_compte_debit': '6278-LCL'},
                           {'credit': Decimal('3.00'),
                            'debit': Decimal('0.00'),
                            'nom_compte': 'Banques',
                            'numero_compte_credit': '512',
                            'numero_compte_debit': ''}],
             'intitule': [' Abonnement LCL Access'],
             'numero_ligne_debut': 134,
             'numero_ligne_fin': 137}
        ])


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(LivreJournalTestCase)
    return suite
